import traceback

from fwd_api.models import SubscriptionRequest, Report, ReportFile
from fwd_api.celery_worker.worker import app
from ..utils import s3 as S3Util
from ..utils.accuracy import (update_temp_accuracy, IterAvg,
                              count_transactions, extract_report_detail_list, calculate_a_request,
                              ReportAccumulateByRequest, create_billing_data)
from ..utils.file import dict2xlsx, save_workbook_file, save_report_to_S3, save_images_to_csv_briefly
from ..utils import time_stuff
from ..utils.redis import RedisUtils
from ..utils.cache import set_cache, get_cache
from django.utils import timezone
from django.db.models import Q
from itertools import chain
import json
import copy
import os

from celery.utils.log import get_task_logger
from fwd import settings

redis_client = RedisUtils()

logger = get_task_logger(__name__)

s3_client = S3Util.MinioS3Client(
    endpoint=settings.S3_ENDPOINT,
    access_key=settings.S3_ACCESS_KEY,
    secret_key=settings.S3_SECRET_KEY,
    bucket_name=settings.S3_BUCKET_NAME
)

def mean_list(l):
    l = [x for x in l if x is not None]
    if len(l) == 0:
        return 0
    return sum(l)/len(l)

@app.task(name='make_a_report_2')
def make_a_report_2(report_id, query_set):
    report_type = query_set.pop("report_type", "accuracy")
    if report_type == "accuracy":
        create_accuracy_report(report_id=report_id, **query_set)
    elif "billing":
        create_billing_report(report_id=report_id, **query_set)
    else:
        raise TypeError("Invalid report type")


def create_accuracy_report(report_id, **kwargs):
    try:
        start_date = timezone.datetime.strptime(kwargs["start_date_str"], '%Y-%m-%dT%H:%M:%S%z')
        end_date = timezone.datetime.strptime(kwargs["end_date_str"], '%Y-%m-%dT%H:%M:%S%z')
        base_query = Q(created_at__range=(start_date, end_date))
        if kwargs["request_id"]:
            base_query &= Q(request_id=kwargs["request_id"])
        if kwargs["redemption_id"]:
            base_query &= Q(redemption_id=kwargs["redemption_id"])
        base_query &= Q(is_test_request=False)
        if isinstance(kwargs["include_test"], str):
            include_test = True if kwargs["include_test"].lower() in ["true", "yes", "1"] else False
            if include_test:
                # base_query = ~base_query
                base_query.children = base_query.children[:-1]
        elif isinstance(kwargs["include_test"], bool):
            if kwargs["include_test"]:
                base_query = ~base_query
        if isinstance(kwargs["subsidiary"], str):
            if kwargs["subsidiary"] and kwargs["subsidiary"].lower().replace(" ", "") not in settings.SUB_FOR_BILLING:
                base_query &= Q(redemption_id__startswith=kwargs["subsidiary"])
        if isinstance(kwargs["is_reviewed"], str):
            if kwargs["is_reviewed"] == "reviewed":
                base_query &= Q(is_reviewed=True)
            elif kwargs["is_reviewed"] == "not reviewed":
                base_query &= Q(is_reviewed=False)

        errors = []
        # Create a placeholder to fill
        accuracy = {"feedback" :{"imei_number":  IterAvg(),
                                "purchase_date":  IterAvg(),
                                "retailername":  IterAvg(),
                                "sold_to_party":  IterAvg(),
                                "invoice_no": IterAvg()},
                    "reviewed" :{"imei_number":  IterAvg(),
                                "purchase_date":  IterAvg(),
                                "retailername":  IterAvg(),
                                "sold_to_party":  IterAvg(),
                                "invoice_no": IterAvg()},
                    "acumulated":{"imei_number":  IterAvg(),
                                "purchase_date":  IterAvg(),
                                "retailername":  IterAvg(),
                                "sold_to_party":  IterAvg(),
                                "invoice_no": IterAvg()}
                    }  # {"imei": {"acc": 0.1, count: 1}, ...}
        time_cost = {"invoice":  IterAvg(),
                     "imei":  IterAvg()}
        bad_image_list = []
        number_images = 0
        number_bad_images = 0
        review_progress = []
        # TODO: Multithreading
        # Calculate accuracy, processing time, ....Then save.
        subscription_requests = SubscriptionRequest.objects.filter(base_query).order_by('created_at')
        report: Report = Report.objects.filter(report_id=report_id).first()
        # TODO: number of transaction by doc type
        num_request = 0
        report_files = []
        report_engine = ReportAccumulateByRequest(report.subsidiary)
        for request in subscription_requests:
            if request.status != 200 or not (request.reviewed_result or request.feedback_result):
                # Failed requests or lack of reviewed_result/feedback_result
                continue
            request_att, _report_files, _atts = calculate_a_request(report, request)
            report_files += _report_files
            report_engine.add(request, _report_files, report)
            request.feedback_accuracy = {"imei_number": mean_list(request_att["acc"]["feedback"].get("imei_number", [None])),
                                         "purchase_date": mean_list(request_att["acc"]["feedback"].get("purchase_date", [None])),
                                         "retailername": mean_list(request_att["acc"]["feedback"].get("retailername", [None])),
                                         "sold_to_party": mean_list(request_att["acc"]["feedback"].get("sold_to_party", [None])),
                                         "invoice_no": mean_list(request_att["acc"]["feedback"].get("invoice_no", [None]))}
            request.reviewed_accuracy = {"imei_number": mean_list(request_att["acc"]["reviewed"].get("imei_number", [None])),
                                         "purchase_date": mean_list(request_att["acc"]["reviewed"].get("purchase_date", [None])),
                                         "retailername": mean_list(request_att["acc"]["reviewed"].get("retailername", [None])),
                                         "sold_to_party": mean_list(request_att["acc"]["reviewed"].get("sold_to_party", [None])),
                                         "invoice_no": mean_list(request_att["acc"]["reviewed"].get("invoice_no", [None]))}
            rq_accuracy = []

            for i, _att in enumerate(_atts):
                if _report_files[i].bad_image_reason in settings.ACC_EXCLUDE_RESEASONS:
                    continue
                if sum(len(value_list) for value_list in _att["acc"]["reviewed"].values()):
                    rq_accuracy += list(chain(*_att["acc"]["reviewed"].values()))
                elif sum(len(value_list) for value_list in _att["acc"]["feedback"].values()):
                    rq_accuracy += list(chain(*_att["acc"]["feedback"].values()))
                    
            request.is_required = False
            if len(rq_accuracy) > 0:
                request.raw_accuracy = mean_list(rq_accuracy)
                request.is_required = request.raw_accuracy < settings.NEED_REVIEW if isinstance(request.raw_accuracy, float) else False # No need to review by default 

            request.save()

            number_images += request_att["total_images"]
            number_bad_images += request_att["bad_images"]
            bad_image_list += request_att["bad_image_list"]
            update_temp_accuracy(accuracy["feedback"], request_att["acc"]["feedback"], keys=settings.FIELD)
            update_temp_accuracy(accuracy["reviewed"], request_att["acc"]["reviewed"], keys=settings.FIELD)
            update_temp_accuracy(accuracy["acumulated"], request_att["acc"]["acumulated"], keys=settings.FIELD)

            time_cost["imei"].add(request_att["time_cost"].get("imei", []))
            time_cost["invoice"].add(request_att["time_cost"].get("invoice", []))

            errors += request_att["err"]
            num_request += 1
            review_progress += request_att.get("is_reviewed", [])

        report_fine_data, _save_data = report_engine.save(report.report_id, kwargs.get("is_daily_report", False), kwargs["include_test"])
        transaction_att = count_transactions(start_date, end_date, report.subsidiary)
        # Do saving process
        report.number_request = num_request
        report.number_images = number_images
        report.number_imei = time_cost["imei"].count
        report.number_invoice = time_cost["invoice"].count
        report.number_bad_images = number_bad_images
        # FIXME: refactor this data stream for endurability
        report.average_OCR_time = {"invoice": _save_data["report"]["file_average_processing_time"]["invoice"], "imei": _save_data["report"]["file_average_processing_time"]["imei"],
                                   "invoice_count": _save_data["report"]["file_average_processing_time"]["invoice_count"], "imei_count": _save_data["report"]["file_average_processing_time"]["imei_count"]}

        report.average_OCR_time["invoice"] = 0 if report.average_OCR_time["invoice"] is None else report.average_OCR_time["invoice"]
        report.average_OCR_time["imei"] = 0 if report.average_OCR_time["imei"] is None else report.average_OCR_time["imei"]
        report.average_OCR_time["avg"] = (report.average_OCR_time["invoice"]*report.average_OCR_time["invoice_count"] + report.average_OCR_time["imei"]*report.average_OCR_time["imei_count"])/(
            report.average_OCR_time["imei_count"] + report.average_OCR_time["invoice_count"]) if (report.average_OCR_time["imei_count"] + report.average_OCR_time["invoice_count"]) > 0 else None
        report.number_imei_transaction = transaction_att.get("imei", 0)
        report.number_invoice_transaction = transaction_att.get("invoice", 0)

        acumulated_acc = {"feedback": {},
                          "reviewed": {},
                          "acumulated": {}}
        for acc_type in ["feedback", "reviewed", "acumulated"]:
            avg_acc = IterAvg()
            for key in settings.FIELD:
                acumulated_acc[acc_type][key] = accuracy[acc_type][key]()
                acumulated_acc[acc_type][key + "_count"] = accuracy[acc_type][key].count
                avg_acc.add_avg(acumulated_acc[acc_type][key], acumulated_acc[acc_type][key+"_count"])
            acumulated_acc[acc_type]["avg"] = avg_acc()

        report.feedback_accuracy = _save_data["report"]["feedback_accuracy"]
        report.reviewed_accuracy = _save_data["report"]["reviewed_accuracy"]
        report.combined_accuracy = _save_data["report"]["average_accuracy_rate"]

        report.num_reviewed = review_progress.count(1)
        report.num_not_reviewed = review_progress.count(0)
        report.num_no_reviewed = review_progress.count(-1)

        report.errors = "|".join(errors)
        report.status = "Ready"

        detail_file_name = "detail_" + report.report_id + ".xlsx"
        dashboard_file_name = "overview_" + report.report_id + ".xlsx"
        report.S3_file_name = os.path.join("report", report.report_id, detail_file_name)
        report.S3_dashboard_file_name = os.path.join("report", report.report_id, dashboard_file_name)
        report.save()

        # Save a list of bad images to csv file for debugging
        save_images_to_csv_briefly(report.report_id, bad_image_list)
        # Saving a xlsx file
        data = extract_report_detail_list(report_files, lower=True)
        data_workbook = dict2xlsx(data, _type='report_detail')
        local_workbook = save_workbook_file(detail_file_name, report, data_workbook)
        s3_key = save_report_to_S3(report.report_id, local_workbook, 5)
        # Save overview dashboard
        # multiple accuracy by 100
        save_data = copy.deepcopy(_save_data)
        review_key = "review_progress"
        for i, dat in enumerate(report_fine_data):
            report_fine_data[i][review_key] = report_fine_data[i][review_key]*100
            keys = [x for x in list(dat.keys()) if "accuracy" in x.lower()]
            keys_percent = "images_quality"
            for x_key in report_fine_data[i][keys_percent].keys():
                if "percent" not in x_key:
                    continue
                report_fine_data[i][keys_percent][x_key] = report_fine_data[i][keys_percent][x_key]*100
            for key in keys:
                if report_fine_data[i][key]:
                    for x_key in report_fine_data[i][key].keys():
                        report_fine_data[i][key][x_key] = report_fine_data[i][key][x_key]*100 if report_fine_data[i][key][x_key] is not None else None
        data_workbook = dict2xlsx(report_fine_data, _type='report')
        if kwargs["is_daily_report"]:
            overview_filename = kwargs["subsidiary"] + "_" + kwargs["report_overview_duration"] + ".xlsx"
            local_workbook = save_workbook_file(overview_filename, report, data_workbook, settings.OVERVIEW_REPORT_ROOT)
            set_cache(overview_filename.replace(".xlsx", ""), save_data)
        else:
            overview_filename = dashboard_file_name
            local_workbook = save_workbook_file(overview_filename, report, data_workbook)
        s3_key = save_report_to_S3(report.report_id, local_workbook)

    except IndexError as e:
        print(e)
        traceback.print_exc()
        print("NotFound request by report id, %d", report_id)
    except Exception as e:
        print("[ERROR]: an error occured while processing report: ", report_id)
        traceback.print_exc()
        return 400


def create_billing_report(report_id, **kwargs):
    try:
        start_date = timezone.datetime.strptime(
            kwargs["start_date_str"], '%Y-%m-%dT%H:%M:%S%z')
        end_date = timezone.datetime.strptime(
            kwargs["end_date_str"], '%Y-%m-%dT%H:%M:%S%z')
        base_query = Q(created_at__range=(start_date, end_date))
        base_query &= Q(is_test_request=False)

        subscription_requests = SubscriptionRequest.objects.filter(
            base_query).order_by('created_at')
        report: Report = Report.objects.filter(report_id=report_id).first()
        billing_data = create_billing_data(subscription_requests)
        report.number_request = len(subscription_requests)
        report.number_images = len(billing_data)
        
        acumulated_acc = {"feedback": {},
                          "reviewed": {},
                          "acumulated": {}}
        for acc_type in ["feedback", "reviewed", "acumulated"]:
            for key in settings.FIELD:
                acumulated_acc[acc_type][key] = None
                acumulated_acc[acc_type][key + "_count"] = None
            acumulated_acc[acc_type]["avg"] = None
        
        report.feedback_accuracy = acumulated_acc["feedback"]
        report.reviewed_accuracy = acumulated_acc["reviewed"]
        report.combined_accuracy = acumulated_acc["acumulated"]

        report.average_OCR_time = {"invoice": None, "imei": None,
                                   "invoice_count": None, "imei_count": None, "avg": None}
        report.status = "Ready"
        detail_file_name = "detail_" + report.report_id + ".xlsx"
        dashboard_file_name = "overview_" + report.report_id + ".xlsx"
        report.S3_file_name = os.path.join("report", report.report_id, detail_file_name)
        report.S3_dashboard_file_name = os.path.join("report", report.report_id, dashboard_file_name)
        report.save()
        data_workbook = dict2xlsx(billing_data, _type='billing_report')
        local_workbook = save_workbook_file(detail_file_name, report, data_workbook)
        s3_key = save_report_to_S3(report.report_id, local_workbook)
    except IndexError as e:
        print(e)
        traceback.print_exc()
        print("NotFound request by report id, %d", report_id)
    except Exception as e:
        print("[ERROR]: an error occured while processing report: ", report_id)
        traceback.print_exc()
        return 400
