import { t } from '@lingui/macro';
import { Button, Result, Typography } from 'antd';
import { AxiosError } from 'axios';
import * as React from 'react';
import { NavigateFunction, useNavigate } from 'react-router-dom';

interface WithRouteProps {
  navigate: NavigateFunction;
}

function withRouter<T extends WithRouteProps = WithRouteProps>(
  WrappedComponent: React.ComponentType<T>,
) {
  const displayName =
    WrappedComponent.displayName || WrappedComponent.name || 'Component';

  const ComponentWithRouter = (props: Omit<T, keyof WithRouteProps>) => {
    const themeProps = { navigate: useNavigate() };

    return <WrappedComponent {...themeProps} {...(props as T)} />;
  };

  ComponentWithRouter.displayName = `withRouter(${displayName})`;

  return ComponentWithRouter;
}

interface ErrorBoundaryProps extends WithRouteProps {
  children?: React.ReactNode;
  imperativeError?: AxiosError | null;
  resetImperativeError?: () => void;
}
type ErrorBoundaryState =
  | { error: null; errorInfo: null }
  | { error: AxiosError; errorInfo: null }
  | { error: Error; errorInfo: React.ErrorInfo };

class ErrorBoundary extends React.Component<
  ErrorBoundaryProps,
  ErrorBoundaryState
> {
  constructor(props: ErrorBoundaryProps) {
    super(props);
    this.state = {
      error: null,
      errorInfo: null,
    };

    this.resetError = this.resetError.bind(this);
  }

  resetError() {
    this.setState({
      error: null,
      errorInfo: null,
    });
  }

  componentDidCatch(error: Error, errorInfo: React.ErrorInfo): void {
    this.setState({
      error,
      errorInfo,
    });
  }

  componentWillReceiveProps(nextProps: Readonly<ErrorBoundaryProps>): void {
    if (nextProps.imperativeError) {
      this.setState({
        error: nextProps.imperativeError,
        errorInfo: null,
      });
    } else {
      this.resetError();
    }
  }

  renderServerInternalError() {
    return (
      <Result
        status='500'
        title='500'
        subTitle={t`Sorry, something went wrong.`}
        extra={
          <Button
            type='primary'
            onClick={() => {
              this.resetError();
              this.props.navigate('/dashboard');
            }}
          >{t`Back to Dashboard`}</Button>
        }
      >
        <Typography.Paragraph>
          <Typography.Text strong>
            {this.state.error?.toString()}
          </Typography.Text>
        </Typography.Paragraph>
        <Typography.Paragraph>
          <Typography.Text style={{ whiteSpace: 'pre-wrap' }}>
            {t`Something went wrong.`}
          </Typography.Text>
        </Typography.Paragraph>
      </Result>
    );
  }

  renderServerMaintainanceError() {
    return (
      <Result
        status='500'
        title='503'
        subTitle={t`Service temporarily unavailable.`}
        extra={
          <Button
            type='primary'
            onClick={this.props.resetImperativeError}
          >{t`Retry`}</Button>
        }
      />
    );
  }

  render(): React.ReactNode {
    if (this.state.error instanceof AxiosError) {
      return this.renderServerMaintainanceError();
    }

    if (this.state.error) {
      return this.renderServerInternalError();
    }

    return this.props.children;
  }
}

export default withRouter(ErrorBoundary);
