import type { TableColumnsType } from 'antd';
import { Table } from 'antd';
import React from 'react';
import {
  ensureMax,
  ensureMin,
  formatNumber,
  formatPercent,
  numberWithCommas,
} from 'utils/metric-format';

interface DataType {
  key: React.Key;
  subSidiaries: string;
  extractionDate: string;
  snOrImeiNumber: number;
  invoiceNumber: number;
  totalImages: number;
  successfulNumber: number;
  successfulPercentage: number;
  badNumber: number;
  badPercentage: number;
  snImeiAAR: number; // AAR: Average Accuracy Rate
  purchaseDateAAR: number;
  retailerNameAAR: number;
  invoiceNumberAAR: number;
  snImeiAPT: number; // APT: Average Processing Time
  invoiceAPT: number;
  snImeiTC: number; // TC: transaction count
  invoiceTC: number;
  reviewProgress: number;
}

const columns: TableColumnsType<DataType> = [
  {
    title: 'Subsidiary',
    dataIndex: 'subSidiaries',
    key: 'subSidiaries',
    width: '100px',
    render: (_, record) => {
      if (record.subSidiaries === '+') return '';
      return String(record.subSidiaries).toUpperCase();
    },
    filters: [
      { text: 'SEAO', value: 'SEAO' },
      { text: 'SEAU', value: 'SEAU' },
      { text: 'SESP', value: 'SESP' },
      { text: 'SME', value: 'SME' },
      { text: 'SEPCO', value: 'SEPCO' },
      { text: 'TSE', value: 'TSE' },
      { text: 'SEIN', value: 'SEIN' },
    ],
    filterMode: 'menu',
    onFilter: (value: string, record) =>
      record.subSidiaries.includes(String(value).toUpperCase()),
  },
  {
    title: 'OCR extraction date',
    dataIndex: 'extractionDate',
    key: 'extractionDate',
    width: 180,
    render: (_, record) => {
      if (record.extractionDate.includes('Subtotal'))
        return (
          <span style={{ fontWeight: 'bold' }}>{record.extractionDate}</span>
        );
      return record.extractionDate;
    },
    filters: [{ text: 'Subtotal', value: 'Subtotal' }],
    filterMode: 'menu',
    onFilter: (value: string, record) => record.extractionDate.includes(value),
  },
  {
    title: 'Total OCR Received (Image Count)',
    dataIndex: 'totalOCRReceived',
    key: 'totalOCRReceived',
    width: '130px',
    className: 'hide-border-right',
    render: (_, record) => {
      const value = record.snImeiTC + record.invoiceTC;
      return <span>{value ? numberWithCommas(value) : '-'}</span>;
    },
  },
  {
    title: '',
    children: [
      {
        title: 'SN/IMEI',
        dataIndex: 'snImeiTC',
        key: 'snImeiTC',
        width: '50px',
        className: 'show-border-left',
        render: (_, record) => {
          const value = record.snImeiTC;
          return <span>{value ? numberWithCommas(value) : '-'}</span>;
        },
      },
      {
        title: 'Invoice',
        dataIndex: 'invoiceTC',
        key: 'invoiceTC',
        width: '50px',
        render: (_, record) => {
          const value = record.invoiceTC;
          return <span>{value ? numberWithCommas(value) : '-'}</span>;
        },
      },
    ],
  },
  {
    title: 'Total OCR Redemptions (Image Count)',
    dataIndex: 'totalImages',
    key: 'totalImages',
    width: '130px',
    className: 'hide-border-right',
    render: (_, record) => {
      const value = record.totalImages;
      return <span>{value ? numberWithCommas(value) : '-'}</span>;
    },
  },
  {
    title: '',
    children: [
      {
        title: 'SN/IMEI',
        dataIndex: 'snOrImeiNumber',
        key: 'snOrImeiNumber',
        width: '50px',
        className: 'show-border-left',
        render: (_, record) => {
          const value = record.snOrImeiNumber;
          return <span>{value ? numberWithCommas(value) : '-'}</span>;
        },
      },
      {
        title: 'Invoice',
        dataIndex: 'invoiceNumber',
        key: 'invoiceNumber',
        width: '50px',
        render: (_, record) => {
          const value = record.invoiceNumber;
          return <span>{value ? numberWithCommas(value) : '-'}</span>;
        },
      },
    ],
  },
  {
    title: 'Image Quality',
    children: [
      {
        title: 'Successful',
        dataIndex: 'successfulNumber',
        key: 'successfulNumber',
        width: '50px',
        render: (_, record) => {
          const value = record.successfulNumber;
          return <span>{value ? numberWithCommas(value) : '-'}</span>;
        },
      },
      {
        title: 'Successful (%)',
        dataIndex: 'successfulPercentage',
        key: 'successfulPercentage',
        width: '120px',
        render: (_, record) => {
          return <span>{formatPercent(record.successfulPercentage)}</span>;
        },
      },
      {
        title: 'Bad',
        dataIndex: 'badNumber',
        key: 'badNumber',
        width: '30px',
        render: (_, record) => {
          const value = record.badNumber;
          return <span>{value ? numberWithCommas(value) : '-'}</span>;
        },
      },
      {
        title: 'Bad (%)',
        dataIndex: 'badPercentage',
        key: 'badPercentage',
        width: '80px',
        render: (_, record) => {
          const isAbnormal = ensureMax(record.badPercentage, 0.1);
          return (
            <span style={{ color: isAbnormal ? 'red' : '' }}>
              {formatPercent(record.badPercentage)}
            </span>
          );
        },
      },
    ],
  },

  {
    title: 'Average accuracy rate (%) \n(※ character-based)',
    children: [
      {
        title: 'SN/IMEI',
        dataIndex: 'snImeiAAR',
        key: 'snImeiAAR',
        width: '130px',
        render: (_, record) => {
          const isAbnormal = ensureMin(record.snImeiAAR, 0.98);
          return (
            <span style={{ color: isAbnormal ? 'red' : '' }}>
              {formatPercent(record.snImeiAAR)}
            </span>
          );
        },
      },
      {
        title: 'Purchase date',
        dataIndex: 'purchaseDateAAR',
        key: 'purchaseDateAAR',
        width: '130px',
        render: (_, record) => {
          const isAbnormal = ensureMin(record.purchaseDateAAR, 0.98);
          return (
            <span style={{ color: isAbnormal ? 'red' : '' }}>
              {formatPercent(record.purchaseDateAAR)}
            </span>
          );
        },
      },
      {
        title: 'Retailer name',
        dataIndex: 'retailerNameAAR',
        key: 'retailerNameAAR',
        width: '130px',
        render: (_, record) => {
          const isAbnormal = ensureMin(record.retailerNameAAR, 0.98);
          return (
            <span style={{ color: isAbnormal ? 'red' : '' }}>
              {formatPercent(record.retailerNameAAR)}
            </span>
          );
        },
      },
      {
        title: 'Invoice number',
        dataIndex: 'invoiceNumberAAR',
        key: 'invoiceNumberAAR',
        width: '130px',
        render: (_, record) => {
          const isAbnormal = ensureMin(record.invoiceNumberAAR, 0.98);
          return (
            <span style={{ color: isAbnormal ? 'red' : '' }}>
              {formatPercent(record.invoiceNumberAAR)}
            </span>
          );
        },
      },
    ],
  },
  {
    title: 'Average Processing Time Per Image (Seconds)',
    children: [
      {
        title: 'SN/IMEI',
        dataIndex: 'snImeiAPT',
        key: 'snImeiAPT',
        render: (_, record) => {
          const isAbnormal = ensureMax(record.snImeiAPT, 2);
          return (
            <span style={{ color: isAbnormal ? 'red' : '' }}>
              {formatNumber(record?.snImeiAPT)}
            </span>
          );
        },
      },
      {
        title: 'Invoice',
        dataIndex: 'invoiceAPT',
        key: 'invoiceAPT',
        render: (_, record) => {
          const isAbnormal = ensureMax(record.invoiceAPT, 2);
          return (
            <span style={{ color: isAbnormal ? 'red' : '' }}>
              {formatNumber(record?.invoiceAPT)}
            </span>
          );
        },
      },
    ],
  },
  {
    title: 'Review Progress',
    dataIndex: 'review_progress',
    key: 'review_progress',
    width: '100px',
    render: (_, record) => {
      return (
        <span>
          {formatPercent(record.reviewProgress) === '-'
            ? 0
            : formatPercent(record.reviewProgress)}
        </span>
      );
    },
  },
];

interface ReportOverViewTableProps {
  isLoading: boolean;
  data: any;
}

const ReportOverViewTable: React.FC<ReportOverViewTableProps> = ({
  isLoading,
  data,
}) => {
  const overviewDataResponse = data as any;
  const dataSubsRows = overviewDataResponse?.overview_data.map(
    (item, index) => {
      return {
        key: index,
        subSidiaries: item.subs,
        extractionDate: item.extraction_date,
        totalImages: item.total_images,
        snOrImeiNumber: item.num_imei,
        invoiceNumber: item.num_invoice,
        successfulNumber: item.images_quality.successful,
        successfulPercentage: item.images_quality.successful_percent,
        badNumber: item.images_quality.bad,
        badPercentage: item.images_quality.bad_percent,
        snImeiAAR: item.average_accuracy_rate.imei_number,
        purchaseDateAAR: item.average_accuracy_rate.purchase_date,
        retailerNameAAR: item.average_accuracy_rate.retailername,
        invoiceNumberAAR: item.average_accuracy_rate.invoice_no,
        snImeiAPT: item.file_average_processing_time.imei,
        invoiceAPT: item.file_average_processing_time.invoice,
        snImeiTC: item.usage.imei,
        invoiceTC: item.usage.invoice,
        reviewProgress: item.review_progress,
      };
    },
  );

  return (
    <div>
      <Table
        loading={isLoading}
        columns={columns}
        dataSource={dataSubsRows}
        bordered
        size='small'
        scroll={{ x: 2000 }}
        pagination={false}
      />
    </div>
  );
};

export default ReportOverViewTable;
