import { PlusOutlined } from '@ant-design/icons';
import { t } from '@lingui/macro';
import { Button, DatePicker, Form, Modal, Select } from 'antd';
import { SbtPageHeader } from 'components/page-header';
import ReportTable from 'components/report-detail/report-table';
import { Dayjs } from 'dayjs';
import { useMakeReport } from 'queries/report';
import { useState } from 'react';

export interface ReportFormValues {
  dateRange: [Dayjs, Dayjs];
  subsidiary: string;
  reportType: string;
}

const DEFAULT_SUBSIDIARY_OPTIONS = [
    { value: 'SEAO', label: 'SEAO' },
    { value: 'SEAU', label: 'SEAU' },
    { value: 'SESP', label: 'SESP' },
    { value: 'SME', label: 'SME' },
    { value: 'SEPCO', label: 'SEPCO' },
    { value: 'TSE', label: 'TSE' },
    { value: 'SEIN', label: 'SEIN' },
  ];

const ReportsPage = () => {
  const [form] = Form.useForm<ReportFormValues>();
  const [isModalOpen, setIsModalOpen] = useState(false);
  const makeReportMutation = useMakeReport();
  const [subsidiaryOptions, setSubsidiaryOptions] = useState(DEFAULT_SUBSIDIARY_OPTIONS);

  const onReportTypeChange = (value: string) => {
    if (value === 'billing') {
      setSubsidiaryOptions([DEFAULT_SUBSIDIARY_OPTIONS[0]]);
      form.setFieldValue('subsidiary', 'SEAO');
    } else if (value === 'payment') {
    } else {
      setSubsidiaryOptions(DEFAULT_SUBSIDIARY_OPTIONS);
    }
  };

  const showModal = () => {
    setIsModalOpen(true);
  };

  const handleOk = () => {
    form
      .validateFields()
      .then((values) => {
        makeReportMutation
          .mutateAsync({
            end_date: values.dateRange[1].format('YYYY-MM-DD'),
            start_date: values.dateRange[0].format('YYYY-MM-DD'),
            subsidiary: values.subsidiary,
            report_type: values.reportType,
          })
          .then((data) => {
            if (!!data && data?.report_id) {
              setIsModalOpen(false);
              window.location.reload();
            }
          })
          .catch((error) => {
            console.log(error);
          });
      })
      .catch((info) => {
        console.log('Validate Failed:', info);
      });
  };

  const handleCancel = () => {
    setIsModalOpen(false);
  };

  return (
    <>
      <SbtPageHeader
        title={t`Reports`}
        extra={
          <Button
            size='large'
            type='primary'
            onClick={showModal}
            icon={<PlusOutlined />}
          >
            {t`New Report`}
          </Button>
        }
      />

      <ReportTable />

      <Modal
        title={t`Create New Report`}
        open={isModalOpen}
        onOk={() => {
          handleOk();
        }}
        onCancel={handleCancel}
      >
        <Form form={form}
          style={{
            marginTop: 30,
          }}
        >
          <Form.Item
            name='dateRange'
            label={t`Date (GMT+8)`}
            rules={[
              {
                required: true,
                message: 'Please select a date range',
              },
            ]}
            style={{
              marginBottom: 10,
            }}
          >
            <DatePicker.RangePicker />
          </Form.Item>

          <Form.Item
            name='subsidiary'
            label={t`Subsidiary`}
            rules={[
              {
                required: true,
                message: 'Please select a subsidiary',
              },
            ]}
            initialValue={'SEAO'}
          >
            <Select
              placeholder='Select a subsidiary'
              style={{ width: 200 }}
              options={subsidiaryOptions}
            />
          </Form.Item>

          <Form.Item
            name='reportType'
            label={t`Report Type`}
            rules={[
              {
                required: true,
                message: 'Please select a type',
              },
            ]}
            initialValue={'accuracy'}
          >
            <Select
              onSelect={onReportTypeChange}
              placeholder='Select a report type'
              style={{ width: 200 }}
              options={[
                { value: 'billing', label: 'Billing' },
                { value: 'accuracy', label: 'Accuracy' },
              ]}
            />
          </Form.Item>
        </Form>
      </Modal>
    </>
  );
};

export default ReportsPage;
