import { DownloadOutlined, ArrowLeftOutlined } from '@ant-design/icons';
import { t } from '@lingui/macro';
import {
  Button,
  Space,
  Tooltip,
  Typography,
} from 'antd';
import { SbtPageHeader } from 'components/page-header';
import { Dayjs } from 'dayjs';
import { ReportDetailList, ReportItemDetail } from 'models';
import { useReportDetailList } from 'queries/report';
import { useEffect, useState } from 'react';
import { useParams } from 'react-router-dom';
import { downloadReport } from 'request/report';
import styled from 'styled-components';
import { SheetViewer } from "react-office-viewer";
import { datetimeStrToDate } from 'utils/time';

export interface ReportFormValues {
  dateRange: [Dayjs, Dayjs];
  includeTest: string;
}

const ReportContainer = styled.div`
  margin-bottom: 16px;
`;

const HeaderContainer = styled(Space)`
  display: flex;
  flex-direction: row;\
  gap: 16px;
  margin-bottom: 16px;
`;

const ReportDetail = () => {
  const [error, setError] = useState(null);
  const [fileObject, setFileObject] = useState(null);
  const [pagination, setPagination] = useState({
    page: 1,
    page_size: 10,
  });
  const { id } = useParams<{ id: string }>();
  const { isLoading, data } = useReportDetailList({
    report_id: id,
    page: pagination.page,
  });
  const report_data = data as ReportDetailList;
  const handleDownloadReport = async () => {
    const {file, filename} = await downloadReport(id, "detail");
    const anchorElement = document.createElement('a');
    anchorElement.href = URL.createObjectURL(file);
    anchorElement.download = filename;

    document.body.appendChild(anchorElement);
    anchorElement.click();

    // Clean up
    document.body.removeChild(anchorElement);
    URL.revokeObjectURL(anchorElement.href);
  };

  // Download and show report
  useEffect(() => {
    try {
      downloadReport(id, "detail", (fileDetails) => {
        if (!fileDetails?.file) {
          setError("The report has not been ready to preview.");
        }
        var blob = new Blob(
            [fileDetails.file],
            {type: "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet;base64,"}
        );
        let blobUrl = URL.createObjectURL(blob);
        setFileObject(blobUrl);
      });
    } catch (error) {
      setError("The report has not been ready to preview.");
      console.log(error);
    }
  }, []);

  const handleBack = () => {
    window.history.back();
  };

  return (
    <>
      <SbtPageHeader
        title={
          <>
            <Tooltip title={t`Back`}>
              <Button
                size='middle'
                type='default'
                icon={<ArrowLeftOutlined />}
                onClick={() => handleBack()}
                style={{
                  marginRight: 10,
                  lineHeight: '1.8',
                  height: 38
                }}
              >
                {t`Back`}
              </Button>
            </Tooltip>
            {t`Report Details`}
          </>
        }
        extra={
          <Button
            size='large'
            type='primary'
            icon={<DownloadOutlined />}
            onClick={handleDownloadReport}
          >
            {t`Download Report`}
          </Button>
        }
      />
      <HeaderContainer>
        <Typography.Title level={5}>
          Subsidiary:{' '}
          <span style={{ fontWeight: '400' }}>
            {report_data?.metadata?.subsidiary}
          </span>
        </Typography.Title>
        <Typography.Title level={5}>
          Start date:{' '}
          <span style={{ fontWeight: '400' }}>
            {datetimeStrToDate(report_data?.metadata?.start_at, 'Asia/Singapore')}
          </span>
        </Typography.Title>
        <Typography.Title level={5}>
          End date:{' '}
          <span style={{ fontWeight: '400' }}>
            {datetimeStrToDate(report_data?.metadata?.end_at, 'Asia/Singapore')}
          </span>
        </Typography.Title>
      </HeaderContainer>
      <ReportContainer>
        {(fileObject && !error) && <>
          <div style={{ width: '100%', marginBottom: '10px' }}>
            <b>Note: </b><i>All changes here will not be saved into server.</i>
          </div>
          <SheetViewer file={fileObject} />
        </>}
        {(!fileObject && !error) && <Typography.Title level={5}>Loading...</Typography.Title>}
        {error && <>
          <Typography.Title level={5}>
          {error}</Typography.Title>
          <div>Try downloading the file:</div>
           <a href={fileObject.url} download={fileObject.name}>
             {fileObject.name}
           </a>
        </>}
      </ReportContainer>
    </>
  );
};

export default ReportDetail;
