import { i18n } from '@lingui/core';
import { t } from '@lingui/macro';
import { notification } from 'antd';
import axios, { AxiosError, HttpStatusCode } from 'axios';
import { getLocale } from 'i18n';
import process from 'process';
import { queryClient } from 'queries';
import { ErrorData } from 'utils/error-handler';
const environment = process.env.NODE_ENV;

const AXIOS_TIMEOUT_MS = 30 * 60 * 1000; // This config sastified long-live upload file request
const EXPIRED_PASSWORD_SIGNAL = 'expired_password';

export const baseURL = environment === 'development' ? 'http://107.120.133.27:9000/api' : '/api';
// export const baseURL = '/api';


export const API = axios.create({
  timeout: AXIOS_TIMEOUT_MS,
  baseURL: baseURL,
  maxContentLength: Number.MAX_SAFE_INTEGER,
  maxBodyLength: Number.MAX_SAFE_INTEGER,
});

export const PublicAPI = axios.create({
  timeout: AXIOS_TIMEOUT_MS,
  baseURL: baseURL,
});

// interceptor to handle locale header
API.interceptors.request.use(
  function (config) {
    try {
      const locale = getLocale();
      if (config && config['headers']) {
        // config['headers']['Accept-Language'] = locale;
      }
    } catch (_) {}

    return config;
  },
  function (error) {
    return Promise.reject(error);
  },
);

PublicAPI.interceptors.request.use(
  function (config) {
    try {
      const locale = getLocale();
      if (config && config['headers']) {
        config['headers']['Accept-Language'] = locale;
      }
    } catch (_) {}

    return config;
  },
  function (error) {
    return Promise.reject(error);
  },
);

// interceptor to inject token to header
API.interceptors.request.use(
  function (config) {
    try {
      const token = localStorage.getItem('sbt-token') || '';
      if (config && config['headers']) {
        config['headers']['Authorization'] = `${JSON.parse(token)}`;
      }
    } catch (_) {}

    return config;
  },
  function (error) {
    return Promise.reject(error);
  },
);

// interceptor to handle unauthenticated (401) or server error (502)
API.interceptors.response.use(
  function (value) {
    return value;
  },
  function onError(error) {
    if (
      // user is currently not in login page and any API return unauthenticated status
      error instanceof AxiosError &&
      error.response?.status === HttpStatusCode.Unauthorized &&
      window.location.pathname !== '/auth/login'
    ) {
      const isPasswordExpired = (
        error as AxiosError<ErrorData>
      ).response?.data.errors?.some((err) => {
        return [err.code, err.message].includes(EXPIRED_PASSWORD_SIGNAL);
      });
      const isPasswordExpiredOnGlobalState = queryClient.getQueryData<boolean>([
        'isPasswordExpired',
      ]);

      if (isPasswordExpired) {
        if (!isPasswordExpiredOnGlobalState) {
          queryClient.setQueryData(['isPasswordExpired'], isPasswordExpired);
          notification.warning({
            key: EXPIRED_PASSWORD_SIGNAL,
            message: i18n._(
              t`Your current password has expired. Please change your password to continue.`,
            ),
          });
        }
      } else {
        localStorage.removeItem('sbt-token');
        queryClient.setQueryData(['isAuthenticated'], false);
      }
    }

    if (
      error instanceof AxiosError &&
      [HttpStatusCode.ServiceUnavailable, HttpStatusCode.BadGateway].includes(
        error.response?.status,
      )
    ) {
      queryClient.setQueryData(['imperativeError'], error);
    }

    return Promise.reject(error);
  },
);
